require "gaebot/version"
require 'yaml'
require 'active_record'

module Gaebot
  class Task
    def initialize(*args, &block)
      @host      = "localhost"
      @database  = "twimaker_game"
      @user_model = nil
      @tweets = []
      instance_eval(&block)
      ActiveRecord::Base.establish_connection(
        :adapter  => 'mysql',
        :host     => @host,
        :username => @username,
        :password => @password,
        :database => @database,
        :encoding => 'utf8'
      )
      @config = Config.load()
    end

    def each_user
      users = Gaebot::User.where("id > :id", {:id => @config[:cursor]}).order(:id).limit(@config[:count])
      last_id = 0
      users.each do |user|
        unless AutoTask.done?(user.id)
          task = AutoTask.new
          task.id = user.id
          yield user, task
          last_id = user.id
        end
      end
      unless users.empty?
        @config[:cursor] = last_id
      else
        @config[:cursor] = 0
      end
    end

    def add_tweet(hash = {})
      @tweets.push(
        :msg          => hash[:msg],
        :token_key    => hash[:token_key],
        :token_secret => hash[:token_secret],
      )
    end

    def output
      JSON.generate(@tweets)
    end

    def save_config
      Config.save(@config)
    end
  end

  class User < ActiveRecord::Base
  end

  class AutoTask < ActiveRecord::Base
    def self.done?(id)
      task = AutoTask.where(:id => id, :date => Time.now.strftime("%Y-%m-%d")).first
      task != nil
    end

    def do()
      if self.id != nil
        self.date = Time.now.strftime("%Y-%m-%d")
        begin
          self.save!
        rescue Exception => e
          AutoTask.update(self.id, :date => self.date)
          #task = AutoTask.where(:id => id).first
          #task.date = self.date
          #task.save!
        end
      end
    end
  end

  class Config
    def self.load()
      if File.exist?("config.yml")
        YAML.load_file("config.yml")
      else
        {:cursor => 0, :count => 20}
      end
    end

    def self.save(config)
      open("config.yml","w") do |f|
        YAML.dump(config, f)
      end
    end
  end

  def self.new(*args, &block)
    Task.new(args, &block)
  end
end
